@@ -44,7 +44,8 @@ typedef enum { /*< underscore_name=gweather_location_level >*/
     GWEATHER_LOCATION_ADM2,
     GWEATHER_LOCATION_CITY,
     GWEATHER_LOCATION_WEATHER_STATION,
-    GWEATHER_LOCATION_DETACHED
+    GWEATHER_LOCATION_DETACHED,
+    GWEATHER_LOCATION_NAMED_TIMEZONE
 } GWeatherLocationLevel;
 
 GWEATHER_EXTERN
@@ -54,9 +55,6 @@ GType gweather_location_get_type (void);
 GWEATHER_EXTERN
 GWeatherLocation      *gweather_location_get_world      (void);
 
-GWEATHER_EXTERN G_DEPRECATED_FOR(gweather_location_get_world)
-GWeatherLocation      *gweather_location_new_world      (gboolean use_regions);
-
 GWEATHER_EXTERN
 GWeatherLocation      *gweather_location_ref            (GWeatherLocation  *loc);
 GWEATHER_EXTERN
@@ -112,6 +110,8 @@ const char            *gweather_location_get_country    (GWeatherLocation  *loc)
 GWEATHER_EXTERN
 GWeatherTimezone      *gweather_location_get_timezone   (GWeatherLocation  *loc);
 GWEATHER_EXTERN
+const char            *gweather_location_get_timezone_str (GWeatherLocation *loc);
+GWEATHER_EXTERN
 GWeatherTimezone     **gweather_location_get_timezones  (GWeatherLocation  *loc);
 GWEATHER_EXTERN
 void                   gweather_location_free_timezones (GWeatherLocation  *loc,
